`timescale 1ns/1ps
module psram_syn_top(
               clk,
               rst_n,
               O_apsram_ck,
               O_apsram_ck_n,
               IO_apsram_dqs,
               O_apsram_dm,
               IO_apsram_dq,
               O_apsram_cs_n,
               init_calib0,
               init_calib1,
               error0,
               error1,

               led
               );

input clk;
input rst_n;

output[3:0]O_apsram_ck;
output[3:0]O_apsram_ck_n;
inout [3:0]IO_apsram_dqs;
inout [31:0]IO_apsram_dq;
output[3:0] O_apsram_dm;
output[3:0] O_apsram_cs_n;

output init_calib0;
output init_calib1;

output error0;
output error1;

output reg [1:0]        led;

wire [63:0] wr_data0/* synthesis syn_keep=1 */;
wire [63:0] rd_data0/* synthesis syn_keep=1 */;
wire [63:0] wr_data1/* synthesis syn_keep=1 */;
wire [63:0] rd_data1/* synthesis syn_keep=1 */;

wire cmd0;
wire cmd_en0;
wire wr_en0;
wire cmd_rdy0;
wire [21:0] addr0;
wire init_done0;
wire [7:0] data_mask0;
wire [9:0] burst_num0;

wire cmd1;
wire cmd_en1;
wire wr_en1;
wire cmd_rdy1;
wire [21:0] addr1;
wire init_done1;
wire [7:0] data_mask1;
wire [9:0] burst_num1;

wire pll_lock;

        reg [31:0]cnt;
        always@(posedge clk or negedge rst_n) begin
            if(!rst_n)
                cnt <= 0;
            else  if(cnt == 32'd40_000_000)
                cnt <= 0;
            else        
                cnt <= cnt + 1'd1;
        end

        always@(posedge clk or negedge rst_n) begin
            if(!rst_n)
                led <= 2'h0;
            else if(cnt >= 32'd20_000_000)
                led <= 2'h3;
            else
                led <= 2'h0;
        end


    Gowin_rPLL your_instance_name(
        .clkout(memory_clk), //output clkout
        .lock(pll_lock), //output lock
        .reset(~rst_n), //input reset
        .clkin(clk) //input clkin
    );


UHS_PSRAM_Memory_Interface_2CH_Top  u_PSRAM_TOP (
               .clk(clk),
               .rst_n(rst_n),
               .memory_clk(memory_clk),
               //.memory_clk_p(memory_clk_p),
               .pll_lock(pll_lock),
               .O_apsram_ck(O_apsram_ck),
               .O_apsram_ck_n(O_apsram_ck_n),
               .IO_apsram_dqs(IO_apsram_dqs),
               .O_apsram_dm(O_apsram_dm),
               .IO_apsram_dq(IO_apsram_dq),
               .O_apsram_cs_n(O_apsram_cs_n),
               .init_calib0(init_calib0),
               .init_calib1(init_calib1),
               .clk_out(clk_x1),
               .cmd0(cmd0), 
               .cmd1(cmd1),
               .wr_en0(wr_en0),
               .wr_en1(wr_en1),
               .cmd_rdy0(cmd_rdy0),
               .cmd_rdy1(cmd_rdy1),
               .burst_num0(burst_num0),
               .burst_num1(burst_num1),
               .cmd_en0(cmd_en0),
               .cmd_en1(cmd_en1),
               .addr0(addr0),
               .addr1(addr1),
               .wr_data0(wr_data0),
               .wr_data1(wr_data1),
               .rd_data0(rd_data0),
               .rd_data1(rd_data1),
               .rd_data_valid0(rd_data_valid0),
               .rd_data_valid1(rd_data_valid1),
               .data_mask0('b0),//(data_mask0),
               .data_mask1('b0)//(data_mask1)
               );

/////////////////////
apsram_test u_test0(
                  .clk(clk_x1),
                  .rst_n(rst_n), 
                  .init_done(init_calib0),
                  .cmd(cmd0),
                  .cmd_en(cmd_en0),
                  .wr_en(wr_en0),
                  .cmd_rdy(cmd_rdy0),
                  .burst_num(burst_num0),
                  .addr(addr0),
                  .wr_data(wr_data0),
                  .rd_data(rd_data0),
                  .rd_data_valid(rd_data_valid0),
                  .data_mask(data_mask0),
                  .error( error0)
                  );


apsram_test u_test1(
                  .clk(clk_x1),
                  .rst_n(rst_n), 
                  .init_done(init_calib1),
                  .cmd(cmd1),
                  .cmd_en(cmd_en1),
                  .wr_en(wr_en1),
                  .cmd_rdy(cmd_rdy1),
                  .burst_num(burst_num1),
                  .addr(addr1),
                  .wr_data(wr_data1),
                  .rd_data(rd_data1),
                  .rd_data_valid(rd_data_valid1),                
                  .data_mask(data_mask1),
                  .error(error1)
                  );





endmodule

