`timescale 1ns/1ps
module apsram_test (
                  clk,
                  rst_n,
                  init_done,
                  addr,
                  cmd,
                  cmd_en,
                  wr_data,
                  wr_en,
                  cmd_rdy,
                  rd_data,
                  rd_data_valid,
                  data_mask,
                  error,
                  burst_num
                  );


input                           clk;
input                           rst_n;
input                           init_done;
input [63:0]                    rd_data;
input                           rd_data_valid;
input                           cmd_rdy;


output                          cmd;
output                          error;
output                          cmd_en;
output[63:0]                    wr_data;
output                          wr_en;
output[21:0]                    addr;
output[7:0]                     data_mask;
output[9:0]                     burst_num;

reg   [9:0]                     burst_rnum;
reg   [9:0]                     burst_wnum;
reg                             wr_en;
reg   [7:0]                     data_mask;
reg   [21:0]                    addr;
reg   [21:0]                    addr_add_w;
reg   [21:0]                    addr_add_r;
reg                             cmd;
reg                             cmd_en;
reg   [63:0]                    wr_data;
reg   [63:0]                    wr_data_add;
reg   [63:0]                    check_data;
reg   [63:0]                    check_data_d0;
reg   [63:0]                    rd_data_d0;
reg                             rd_data_valid_d0;
reg   [7:0]                     next_state;
reg   [7:0]                     curr_state;
reg   [9:0]                     WR_CNT;
reg   [9:0]                     RD_CNT;
reg                             error_d;
reg                             error_d0;
reg                             error_d1;
reg   [9:0]                     cyc_cnt;
reg                             cyc_done;
reg                             cyc_done_d;
//=====test state machine=====

localparam  IDLE                = 8'b00000001;
localparam  WRITE_BURST         = 8'b00000010;
localparam  WRITE_WAITE         = 8'b00000100;
localparam  CMD_WAITE           = 8'b00001000;
localparam  READ_BURST          = 8'b00010000;
localparam  READ_WAITE          = 8'b00100000;
localparam  CYC_DONE_WAITE      = 8'b01000000;
localparam  CYC_DONE            = 8'b10000000;

localparam  MAX_BL              = 10'd128; // 133Mhz 656/4;
                                           // 166Mhz 768/4; 192-3 = 189;
                                           // 200Mhz 768/4;
                                           // 200Mhz WL = 2 984/4;246-3 = 243;
                                           // 233Mhz WL = 2 1024/4;
                                           // WIRTE option limit MAX BL,READ option has no limit;
 
assign      burst_num           = (curr_state == WRITE_BURST) ? burst_wnum:
                                  (curr_state == READ_BURST)  ? burst_rnum:  9'b0;           
     
//localparam  ADDR_RANGE          = 10'h1FF;//15'h7FFF;


   

always@(posedge clk or negedge rst_n)
    if(!rst_n)
        curr_state <= IDLE;
    else 
        curr_state <= next_state;

always@(*)begin
    next_state = curr_state;
    case(curr_state)
        IDLE: 
            if(init_done && cmd_rdy)
            next_state = WRITE_BURST;
                
        WRITE_BURST:
            if(WR_CNT == burst_wnum) 
            next_state = WRITE_WAITE;
        
        WRITE_WAITE: 
                next_state = CMD_WAITE;

        CMD_WAITE:
            if(cmd_rdy)
                next_state = READ_BURST;

        READ_BURST:
            if(RD_CNT == burst_rnum)
            next_state = READ_WAITE;

        READ_WAITE:
            if(cmd_rdy)
            next_state = CYC_DONE_WAITE;

        CYC_DONE_WAITE: 
            if(cyc_done) 
            next_state = CYC_DONE;
            else
            next_state = IDLE;
        
        CYC_DONE:
            next_state = IDLE;
      
        default: next_state = IDLE;
       endcase
      end


//=====BURST WRITE =====

always@(posedge clk or negedge rst_n)
    if(!rst_n)
     WR_CNT <= 'd0;
    else if(curr_state ==IDLE)
     WR_CNT <= 'd0;
    else if(curr_state == WRITE_BURST)
     WR_CNT <= WR_CNT + 1'b1;




//=====BURST READ =====
  
always@(posedge clk or negedge rst_n)
    if(!rst_n)
     RD_CNT <= 'd0;
     else if(curr_state ==IDLE)
     RD_CNT <= 'd0;
    else if(curr_state == READ_BURST)
     RD_CNT <= RD_CNT + 1'b1;



//===== pSRAM CTRL =====
always@(posedge clk or negedge rst_n)
 if(!rst_n)
   begin
   cmd         <= 1'b0;
   cmd_en      <= 1'b0;
   //wr_en       <= 1'b0;
   addr        <=  'b0;
   wr_data     <=  'b0;
   data_mask   <=  'b0;
   addr_add_w  <=  'b0;
   addr_add_r  <=  'b0;
   wr_data_add <=  'b0;
   end
 else if((WR_CNT == 'd0) && (curr_state == WRITE_BURST))
   begin
   cmd         <= 1'b1;
   cmd_en      <= 1'b1;
   addr        <= addr_add_w;
   wr_data     <= wr_data_add;              
   data_mask   <=  'b0;
   addr_add_w  <= addr_add_w + burst_wnum<<1;
   wr_data_add <= wr_data_add + 1'b1;   
   end
 else if((WR_CNT !== 'd0)  && (curr_state == WRITE_BURST))
   begin
   cmd         <= 1'b0;
   cmd_en      <= 1'b0;
   addr        <=  'b0;
   wr_data     <= wr_data_add;              
   data_mask   <=  'b0;  
   addr_add_w  <= addr_add_w;
   wr_data_add <= wr_data_add + 1'b1;  
   end
 else if((RD_CNT == 'd0) && (curr_state == READ_BURST))
   begin
   cmd         <= 1'b0;
   cmd_en      <= 1'b1;
   addr        <= addr_add_r;              
   data_mask   <=  'b0;
   addr_add_r  <= addr_add_r + burst_rnum<<1;
   end
 else if(curr_state == CYC_DONE)
   begin
   cmd         <= 1'b0;
   cmd_en      <= 1'b0;
   addr        <=  'b0;
   wr_data     <=  'b0;
   data_mask   <=  'b0;
   addr_add_w  <=  'b0;
   addr_add_r  <=  'b0;
   wr_data_add <=  'b0;
   end
 else
   begin
   cmd         <= 1'b0;
   cmd_en      <= 1'b0;
   addr        <=  'b0;
   wr_data     <=  'b0;
   data_mask   <=  'b0;
   end

always@(posedge clk or negedge rst_n)
 if(!rst_n)
   wr_en <= 1'b0;
 else if(curr_state == WRITE_BURST)
   wr_en <= 1'b1;
 else 
   wr_en <= 1'b0;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
  begin
   burst_rnum <= 'd1;
   burst_wnum <= 'd1;
  end
 else if(curr_state == CYC_DONE)
  begin
   burst_rnum <= 'd1;
   burst_wnum <= 'd1;
  end
 else if(curr_state == CYC_DONE_WAITE)
  begin
   burst_rnum <= burst_rnum + 'd1;
   burst_wnum <= burst_wnum + 'd1;
  end

always@(posedge clk or negedge rst_n)
 if(!rst_n)
   cyc_cnt <= 10'b0;
 else if(curr_state == CYC_DONE)
   cyc_cnt <= 10'b0;
 else if(curr_state == CYC_DONE_WAITE)
   cyc_cnt <= cyc_cnt + 1'b1;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
   cyc_done <= 1'b0;
 else if(cyc_cnt == MAX_BL && (curr_state == READ_WAITE))
   cyc_done <= 1'b1;
 else if(curr_state == CYC_DONE)
   cyc_done <= 1'b0;

//=====check read_valid and read data=====
always@(posedge clk or negedge rst_n)
 if(!rst_n)
   cyc_done_d <= 1'b0;
 else
   cyc_done_d <= cyc_done;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
    check_data <= 'b0;
 else if((curr_state == IDLE) && cyc_done_d)
    check_data <= 'b0;
 else if(rd_data_valid)
    check_data <= check_data + 1'b1;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
  rd_data_d0 <= 'b0;
 else 
  rd_data_d0 <= rd_data;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
  check_data_d0 <= 'b0;
 else 
  check_data_d0 <= check_data;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
  rd_data_valid_d0 <= 1'b0;
 else 
  rd_data_valid_d0 <= rd_data_valid;

always@(posedge clk or negedge rst_n)
 if(!rst_n)
  error_d0 <= 1'b0;
 else if(rd_data_valid_d0 && (check_data_d0[31:0] !== rd_data_d0[31:0]))
  error_d0 <= 1'b1;
 else 
  error_d0 <= 1'b0;

/*always@(posedge clk or negedge rst_n)
 if(!rst_n)
  error_d1 <= 1'b0;
 else if(rd_data_valid_d0 && (check_data_d0[63:32] !== rd_data_d0[63:32]))
  error_d1 <= 1'b1;
 else 
  error_d1 <= 1'b0;*/


always@(posedge clk or negedge rst_n)
 if(!rst_n)
  error_d <= 1'b0;
 else
  error_d <= error_d0 ;

assign error = error_d;

endmodule



