// *******************************************************************************************************
// **                                                                                                   **
// **    APM_APS3208K-OKUD - 32Mb OPI DDR PSRAM written in Verilog HDL                                  **
// **                                                                                                   **
// *******************************************************************************************************
// **                                                                                                   **
// **                   This information is distributed under license from AP Memory Corp.              **
// **                          COPYRIGHT (c) 2019 AP Memory Technology Corp                             **
// **                                      ALL RIGHTS RESERVED                                          **
// **                                                                                                   **
// **                                                                                                   **
// **                                                                                                   **
// *******************************************************************************************************
// **                                                                                                   **
// **   This information is provided to you for your convenience and use with AP Memory products only.  **
// **                                                                                                   **
// **   THIS INFORMATION IS PROVIDED "AS IS." AP MEMORY MAKES NO REPRESENTATION OR WARRANTIES OF        **
// **   ANY KIND WHETHER EXPRESS OR IMPLIED, WRITTEN OR ORAL, STATUTORY OR OTHERWISE, RELATED TO        **
// **   THE INFORMATION PROVIDED TO YOU, INCLUDING BUT NOT LIMITED TO ITS CONDITION, QUALITY,           **
// **   PERFORMANCE, MERCHANTABILITY, NON-INFRINGEMENT, OR FITNESS FOR PURPOSE.                         **
// **   AP MEMORY IS NOT LIABLE, UNDER ANY CIRCUMSTANCES, FOR SPECIAL, INCIDENTAL OR CONSEQUENTIAL      **
// **   DAMAGES, FOR ANY REASON WHATSOEVER.                                                             **
// **                                                                                                   **
// **   It is your responsibility to ensure that your application meets with your specifications.       **
// **                                                                                                   **
// *******************************************************************************************************
// **                                                                                                   **
// **   Revision       : 2.26                                                                           **
// **   Modified Date  : 12/19/2019                                                                     **
// **   Revision History:                                                                               **
// **                                                                                                   **
// **   09/22/2016:  Initial Release                                                                    **
// **   12/13/2016:  updated tCSP/tCHD, xCE high & DQS, cmd X/Z filtering, VERIF signals,               **
// **                added Fmin error, tCEM to 3.6us, removed 4B                                        **
// **   01/09/2017:  fixed checks for 0 CLK'd CElow                                                     **
// **	10/05/2017:  changed when registers are checked for frequency                                   **
// **	10/18/2017:  changed to table type latency check vs formula                                     **
// **	10/19/2017:  fixed LCr set in first read is VL push out case, adjusted refresh duration min/max **
// **	01/05/2018:  relaxed WL2 requirement                                                            **
// **	03/02/2018:  parameterized HS function, added tPU checks, stop sampling tCLK after 4th clk,     **
// **                added output strength, refresh & halfsleep indicators                              **
// **   08/29/2018:  added additional reset for internal DQS_DO to 0 during CE#                         **
// **   11/08/2018:  added CE reset to TimingCheckAddrEnable logic                                      **
// **   12/05/2018:  parameterized wrap length                                                          **
// **   01/24/2019:  added toggle to turn off tCL/tCH checking after CMD/Addr phase                     **
// **   02/11/2019:  fixed WrtLat reset                                                                 **
// **   11/25/2019:  updated tCPH check and tCEM, added tWPRE/WPST & tDQSH/DQSL checks                  **
// **	12/19/2019:	 add SR0[4:2]=3'b111 for 233MHz LC check and correct CECounter						**
// **                                                                                                   **
// *******************************************************************************************************
// **                                       TABLE OF CONTENTS                                           **
// *******************************************************************************************************
// **---------------------------------------------------------------------------------------------------**
// **   DECLARATIONS                                                                                    **
// **---------------------------------------------------------------------------------------------------**
// **---------------------------------------------------------------------------------------------------**
// **   INITIALIZATION                                                                                  **
// **---------------------------------------------------------------------------------------------------**
// **---------------------------------------------------------------------------------------------------**
// **   CORE LOGIC                                                                                      **
// **---------------------------------------------------------------------------------------------------**
// **   1.01:  Internal Reset Logic                                                                     **
// **   1.02:  Input Data Shifter                                                                       **
// **   1.03:  Bit Clock Counter                                                                        **
// **   1.04:  Instruction Register                                                                     **
// **   1.05:  Address Register                                                                         **
// **   1.06:  Status Register Write                                                                    **
// **   1.07:  Array Write                                                                              **
// **   1.08:  Output Data Shifter                                                                      **
// **   1.09:  Output Data Buffer                                                                       **
// **                                                                                                   **
// **---------------------------------------------------------------------------------------------------**
// **   DEBUG LOGIC                                                                                     **
// **---------------------------------------------------------------------------------------------------**
// **   2.01:  Memory Data Bytes                                                                        **
// **                                                                                                   **
// **---------------------------------------------------------------------------------------------------**
// **   TIMING CHECKS                                                                                   **
// **---------------------------------------------------------------------------------------------------**
// **                                                                                                   **
// *******************************************************************************************************

`timescale 1ns/10ps

`define tCLK4300

`define tDQSCKmin	2 	// fixed 2

`define tHZ 		4.5	// max 5ns @166 or 5.5ns @133
`define tDQSCK		2	// range 2-5ns @166 or 5.5ns @133 (use max for model delivery)
`define tDQSQ		0.4	// max 0.5ns @166 or  or 0.6ns @133
`define tRC		55
module psram_model (xDQ, xDQS, xCEn, xCLK, xCLKn, xDM);

`ifdef tCLK5000
	parameter tCLKmin	= 5;
	parameter tHZmax	= 4.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSCKmax	= 5.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSQmax	= 0.4;			// should be used for timing checks only, and not model output behaviour
`else
`ifdef tCLK6000
	parameter tCLKmin	= 6;
	parameter tHZmax	= 5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSCKmax	= 5.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSQmax	= 0.5;			// should be used for timing checks only, and not model output behaviour
//`else `define tCLK7500
`else
`ifdef tCLK4300
	parameter tCLKmin	= 4.29;
	parameter tHZmax	= 4.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSCKmax	= 5.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSQmax	= 0.4;			// should be used for timing checks only, and not model output behaviour
`else
	parameter tCLKmin	= 7.5;
	parameter tHZmax	= 5.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSCKmax	= 5.5;			// should be used for timing checks only, and not model output behaviour
	parameter tDQSQmax	= 0.6;			// should be used for timing checks only, and not model output behaviour
`endif
`endif
`endif
	parameter tCQLZmax	= 6;
	parameter tCHmin	= 0.45;
	parameter tCHmax	= 0.55;
	parameter tCLmin	= 0.45;
	parameter tCLmax	= 0.55;
	parameter tCEMmax	= 2_500;
	parameter tCEMmin	= 2;
	parameter tDQSH		= 0.4;
	parameter tDQSL		= 0.4;
	parameter tWPRE		= 0.4;
	parameter tWPST		= 0.4;
	parameter tPU		= 150_000;
	parameter tHS		= 4_000;
	parameter tXHS		= 100_000;
	parameter tXPHS		= 60;

   input                xCLK;                           // serial data clock
   input                xCLKn;                          // serial data clock bar
   input                xCEn;                           // chip select - active low
   input                xDM;                            // Data Mask select - active low
   inout                xDQS;                           // Data strobe
   inout [7:0]          xDQ;                            // Data/Address In/out


// *******************************************************************************************************
// **   DECLARATIONS                                                                                    **
// *******************************************************************************************************

   reg  [23:00]         DataShifterI;                   // serial input data shifter
   reg  [15:00]         BitCounter;                     // serial input bit counter
   reg  [07:00]         InstRegister;                   // instruction register
   reg  [23:00]         AddrRegister;                   // 24-bit address register
	parameter row_width 	= 12;
	parameter col_width 	= 10;
	parameter memad_width 	= row_width+col_width;
	parameter memad_size 	= 1 << memad_width;
   reg [row_width-1:0]		RowAddrin;
   reg [col_width-1:0]		ColAddrin;
   reg [row_width-1:0]		RowAddr;
   reg [col_width-1:0]		ColAddr;
   wire [memad_width-1:0]	MemAddr;

   wire                 InstructionREAD;                // decoded instruction byte
   wire                 InstructionRDSR;                // decoded instruction byte
   wire                 InstructionWRSR;                // decoded instruction byte
   wire                 InstructionWRITE;               // decoded instruction byte
   wire                 InstructionGBLRST;              // decoded instruction byte
   wire			Valid_INST;

   reg  [07:00]         MemoryBlock [0:memad_size-1];
   
        parameter VeriOutStr    = 0;	// use 1 if host is using bus keepers
	
   reg  [7:0]           SR[0:7];
   reg	[7:0]		MROut[1:0];
   reg  [2:0]		MA1;
   wire [7:0]           SR0;
   wire [7:0]           SR1;
   wire [7:0]           SR4;
   wire [7:0]           SR6;
   reg  [7:0]           DOUT;
   reg  [7:0]           DOUT_d;
   
`protected
    MTI!#DCJxQ$>rhT]E{t]D';BGo<oK;Jl\YWwY#ugz9cFH=Y>WQk^wR!3wTu'HnKu$rY?OV'WQ?YZ
    VAQoe?OGXHTV.3xBul>KA<8QL?$GrJNDC<oUVEpu-Zu\xG-^vQ#FRi_>^5=>e[so$e2?^3v$]=*7
    ijKJE-eETau-/3Y7u-=~K=uWa}3J_;w@=<[*56FAH$<a=B!t;$2m^<X\#=_ToB!^*;5VCQ-=C?<u
    AETjx,p$vj?^3Y'o!{7ix6QV\Iuz;o~{*KJj-G#nCr?<2,#YCVv^?]\Gr$,]vo'WX$W1+_3sZk\#
    u,r]GAJx#QQ{Zsn<$Z}>vDK={^WU+--=Cv[JWm+{[ZeC1B{wse}uB2,s+=W>vu~Cu}s,<$u[nDH-
    oiNv#Zj]zCAP3_YDUrZAZx<_~Rl?A$@,^^w7~5m[\[#[),3xuGk^]c,0rV<p*:FCk-,YsIYA<
`endprotected

   
   integer		errors, warnings;
   reg			sys_timing_err_input, sys_timing_err_CE, sys_timing_err_data;
   
`protected
    MTI!#DTW!]K~v7HE}<>o?7~xkKY}ZoZ3Q?Ykv[f'}."i^x'YGi5~I~Iar7i}3]QyExZ=;'o$O[u_
    DKz@bj}wG'25X|r7$i]R3TEEraOZ\]@RGx"PY#*zspx[I[o{^oo~t*b:IT,pBrJ7?XpG{w2*#G3@
    Zo1!(oZH-lm['yEXG^aaUE[B_<^2\uI>s-;\?GpK@p\i1,2vorV}v]5!o*YERjCiTm{UnJ^Yz]^a
    [zOlmxzN-Xz+$*~1|7Iu+RiQmxa.kYes"_z_YPa*J@r$K\Kp'}Nqv~D_3nY~l\r~{lT?FR2*-.^G
    B}%Y_B*;o+\:~nVWu*-BirkY,@5uy7ABOk{2eB]7k}[la^BE^j7@Ck]i1=pee9^mQ]GwI1L#527*
    wC7-[*}O-~@l,slHBo]Vo~!i'ETI5i_rO{Q{oA#3{,~Z[B#XTvzoa-@,7sui_llsxw,W}kIv+;[)
    R-IT}nGQB.H7j"KBEEwGB,
`endprotected


   reg				VERIF_trig_read_byte;
   reg				VERIF_trig_write_byte0;
   reg				VERIF_trig_write_byte1;
   
   reg [memad_width-1:0]	VERIF_read_address;
   reg [memad_width-1:0]	VERIF_write_address0;
   reg [memad_width-1:0]	VERIF_write_address1;
   
   reg [7:0]			VERIF_read_value;
   reg [7:0]			VERIF_write_value0;
   reg [7:0]			VERIF_write_value1;
   
   integer		i;

   parameter STOP_ON_ERROR = 0; 	// If set to 1, the model will halt on errors
   parameter sim_RFSH = 1; 		// 1 = model will simulate VL refresh pushout, 0 = no refresh pushout
   wire			rfsh_evnt;	// use this to monitor VL refresh pushout condition
   wire			HS_evnt;	// use this to monitor HS status

`protected
    MTI!#cCel'YC*ID[rVz~[Vx1@mCjZUzzR_|$'k=C,."-U1'Yp'B<<EkriQkvA*27mQQ2$kDxkW^\
    +_Ws*7k4{XzEvfmX,;[-I21WGp'kC#x+XY1e7?jY]JBW\~@T=6BG]<Y>KIHs?nZ1W~>l32,@Zzhg
    aYwDz--!#EKHmwxK^n<QA][prW*I[-1#E@m*;'$Bz_WKHIf=V*pkz$-kYj!5[apI~ZjjGp,iHAW;
    r[vEKZ_fok*p:{UTKFeOBX7?^I^Tz^tOj#p}#BG?VI|$;TlNFl,e2W=Vu[3{CyuB7?o-1^I2X2ux
    -amjQCgU}vze,{'Uow*u{UoP2>D3#1{2ym]B7~*>+,GlU*OH71ij;7DnZejvsc:'Di=VAoIh?}2^
    [};C(?zwWf'QjGQ;DIW]#uk+uBmCiJIC?2xKja2p[Vz^<$T,Qw;osA5TE@z5?Z2r#~DS-nQx}BzD
    9,{xR,p3>N1jlmJjv]-R@-#jx@XrIlHA$upvu$"xxH>^377"!$\5GD@T:[wwDB!E^k7E7!5v<DeJ
    mRxwQR2[<L7o$a$Xx+ll$HK'{BPtZ-,YY1T\S&]Y##G}^O]['\7!JVrgOQ[eu]~reDUJ>RQ]la<B
    e}Ek^5Beh="krwJ73[Iqu}o]}\-aIH]{$xB<XXXGxw@osJ{]T-DZB{@V&UE#[wx+u2[!n&s>CY0<
    ww[=uX-Z$7s9IA{vX^''e7J[\@smBr'_\EnCG=UHECu$E^$=vWa{aG;p&e77#=1'[]E*VpsY}lD\
    ^]sW<En5B\T7XsVp{@xYK7$n=^KWAV\}Xe{jnEn}'y=C5X>AKGR#Uze]??p;D7]r-#w7e{\UE{=r
    >A+Rw\_z_?L5v=k0Yk\B<E5~];;mZD2l?X2[DWsjc(IrD!+a;{I@pzCTx!,xj;5#jP9ExO~2,2nz
    wj1i$=}A\+aCuHe}2=zf@,!Y2jIirXu15!5Txvk$KQrR1;\Il3[A*lGJ-[DX,}A<-VW4<TBA=5\B
    2D<!!A\D^s[o0YvQ3{woA'j!U1JP<T_7d[kp}}u_G{'TRw$pk9HHu{^KV7qB_+p{D7jQ}*$p[E!}
    ^wVIjpD_m@B^{1OD+v!YDDAKv^;=*nO@}-v,@prGrDoT*Oi;YzQ~Yw[^[kBcl}A*\p<AFj>[iUOu
    _Lwt{[]D}4q=>@p7\7m=<J*_rGB=$A!v*$ua<>ZYiOYo2pBa[Y=e=AuRnYi2wZK6GB77q+VumK&5
    -^<C'Vp!H{T6B@JazT32em\*JR7U^*?l>jpB1!XA<XsjGnuZ@>Bo[E]|HVW@nB#WY[J!]o*T,-@C
    aA{ET[nIy)QWTRr7QDEXTYQ~B~emxI7#'Z1T@Eb/H7iu}$Qm$3Yo\{Ie@UD>ZwDC(zR#T1}1@5x_
    AmpH@9K^\1zZ$!n=X[x<-=?zET[\7'KeV}p;>}EHUQwB-@1OBsRG*GI~<X?=_TGv'v'ZY?uj\]ha
    SlXT[vZ5X?w75/B~Y?@,7er=[igf^pUU)-|m-A{}JCk}+-!q!s'<v>QvrE\IRH*C*='21]2Y]]O,
    f%YI1JXBV2RY;El}UK[YHj.7uuTK*;^F#*#*G}T,p1];sz3<CV$A_}--x*,CsazKvET@wI,~Bp#o
    {nVWZD@xz;<YyzWE1<\E!,,^V<v@U{B!jI2B1W^Op]B>\>5Gis]BK[3Q#n}}1l#!;GHjXXS~>IC=
    QOkn^_Jvejw>G_JW7X5"WG<,s;!eA5u'@RXannVn,?7oswQ<IZo=\a\u{_e])l3'iqvnR*W5l'pJ
    pU,5oiW}k}\mmzDK<U.lI7Tbn{pVj7wBQ>e_CpHUIK2XlQHk',]ECul#\QX?Eapj97wA*z5,Rlw7
    $L8s$u<$vsBYao+WB3ID]~?EZ5[klv;C;]-97<YCrE+j:"e-p$yzDnu3''1eTR,UlOz[=iZB5+WF
    DJ9%RmB3!AZIBi7XKo-IG$kUR>WaBQK\D2\$D~eu}>QwYn\@}EUakss<+Up^[zXoIX2#Bq0r%[mK
    Q;a+*mG3')jH<<'nm{*k-$S6|Esk+s2p1ar[1_K1<ydMkwwD25G=~_3~0cK|n7wv^<w\C7Bo:(*n
    XvEZ*Y+*xDGRB_AXlHl,RIozn@t.?*oC3{YEnoYGYCAZ}1TsX]xx?_Bo/,2,\s!oA&.XIx[mer5/
    pV7Q7o$=:+lBohJ=l_{5GpOnC$?IW]CV_!RAw{m.^^2_o>B,M9m'RTJAG~E!~!Jv*<{X;u=7@ZeY
    pQ()}X_{=2uJXOluT-$XZ5{C^D3'VU7RJs2?!vH>rQ>H>-H6GA<;]v_#iY[W1n+5]vH,!\5{2<Jm
    =*7jG\~e#v<veKDEf&c+&3zWEQ_JHs;sRvQo+OEA{+\XxK-=p?5H7$CTJ~e7Ax[X*v1ABr+WEex'
    _rBI!-xvi}kYH-EaT3\Aao5-DwQeoIR<EO@$s@5pi-$$v#n~eLr>Wow'-'b4'BQJAC<CX<3[RGsi
    xOC_HN5[>prs=HZGOE]Ra2OVB$g8kank$Aja$>v'"RYT*L5YnkG[<VR{,<XXGV],!2@*XZVXYiGH
    +m5_K[)?xm_aC~lYE^C$}+jEZj*A>RT=Y\}5WCGCK*}YnZkpkA*&JeUA\}JWZR$z3p*@/$p$1Uoo
    ip*Qi|Z_e7I^;$^\.u[>}!7$B2+{1Gi~29glEzTp@,_#EauGQHAQ$;lc\um7;HaV;zr^]iaA\u5Y
    rEKKXqMvr{o9_j+{?\;XG5lQ{[-~$_X[i1~<'}!rvDHkBm^U-BOsT^YO[uWYnO_;}#'sC?\nUXB1
    /TC?Ux]\{JCK]$2D#T7eA'5,!}$Iz-[l1yeWYXwV@?V{Zu\E3po~a\pnB-Uzp[WpH7}To';rUY?$
    ixk+aelu>\0q!<_T'viEYu,\[GsG\+r@ccID=G]>x5*w\[p!naR5\x!>[VwR*EV1B<HvX@x3u\kR
    ^m8hxn{j*CpI_vVO9eQiTm}p^[3r5rlYAJEiVAlZTZ{-Te7kVMeEW{7[1;-w_=L>v>]G;X=/bOG?
    [B{Yu'}7'z]]=Q+RuG@oG++7@otP2REJs^jvi{1~G5[xkU$ZHtieV#3oleX[al+,Q7/ZHs^O7RRV
    -e[12]j']JxKRpskjU2C-<!Q@!~}VY{u^l1155o1W*eCBwUtX$A}lad+nmmQxkG]2;}Y2+-?>ZX'
    #n][=7IC>,[;rHv]}\3NC~^x0[7U5I!~1^iY}l;Q{N;UD!Bsl5r'ITKQnY-woJH<~7YU{kGvj7cr
    5$B}T*_>O]v_zCpi5G{~,UQU,~X=_CBmAwQqQ-=35~{!nX1OX'elG5UX-^zz?s#vzY3jBKQpBJpJ
    R;<xhJrnEaXlU1u3o@X=U61KOEssB,_VW]G}w[,!ZHyE<'>aV^EA=us7Im,He'*HYxC."o\I+cz<
    =*/pIE?vvG>=k7ks2r{s+Uv1~VZaA}RB~rovi}YsoI}xpoT+vrH.a]e'U=v_kaKUWxvU#RY_R}+a
    7oWl_URw*WIl^zxB=${QR{LfJ5{BDQk7G<rR&E"Oi;>a>aARkUHN}@Wz}V{JlA;$QkxiUA]r!5<5
    xR[7}A]]u$DxD]]~Y{Xwr*#oJG?2IABj^{+r-pX^zzjo4pUB>LCxa~7c^H21'[lCh3lECH,-7]I[
    s>1$}4IcOi>us@Jv~^7-e>KH$XnO\Rp@D_O{OHV1rrJG31TxkR1@inn'lCv#WA-XD3JJE;pG^wo@
    AB3QJ=pGBl,3#*{rbAw;$O!G+5v@j+T=AMYXH-r-lkXBTa2l{eliG5SR--7]3WUkVuK_U;sG_#eC
    UDI!1ua{x_@N}a]u]W~W@][YV$}JBI<j<r??G2m@gV=al&_mU!jw~E_^x['xnn-*yNQHWwQACBBA
    *$D2aw((?>TuOp'[2j@=AV]DB>>\x~~W;_W~AXr~1@]'"O@!--wm$a}VAS$#OYOo,]UBTrGp1Q'\
    mnpTHH$;$ivG\xdv=;@=:jQ#7GHKE|nXR2H5_7<'ZV]Qe[XAE^1_eEOiZ$\@^;/#nsr>_!I*--$>
    wxTS!<EYeTu}IOJ]E^R_-oel.-7ZDXAZ@AouJ,15IFY;UzH\uw,D$kE-lCx!OaTO7oBp;Gr2~>SE
    \}uB~$^\J~u=3]rI[im@-s~H>>7c6h9ap-v^,$onCY[e-BuQi\rR1so35ruAIvXDA27w_OzIW!KT
    GXus1jp3^{~_OVK3p#{;_X17'oaz[vYOYUuUjj}+w2T2s!ev51vJxXukTK]E<Y~hC?@W;VD3\Hvu
    1O[*$-$lO}"wxQiII*G~>lI{Q3^?sr]{w[vs>'x]X'm{X}~q\au=DYJrtsaVGf'ImxO>dC$p#Y1Y
    lLk5ok_?xkG\W7kXH;Y+r?=C7vlae?Ewp<keauK>1WDAo]RnAuXQK#>rmT8=lKGmOVij+\l?Y!=l
    ,@2e4ei][Zr1~lJ7eU<;o]a@7r{]D;{UA_Qa~_JHRXtdeTB;.mn}r=\V7nXx,[.5$Bie'xoB*+o^
    wu'1'j-$a]-UXw=3E'B>H=zK[DoGmHr~x=K@G--zeRQ!la3=W5vG5j3U5'u*ZjuBJwB#oCxyQE#o
    -na<Xv3-#H=T#R\H\7v~5io_s*?JH*>p}e+XXHn7C[YB@I$U3=u#n}B;_kG+}O3=OXQpBB#>E+mH
    ,$mYUX2rfIDwRk5p}k}@'re1Dn$-kcdQmw<j{QEM5]l3zCm#J5O}T>u7G8;^-},sAxT]DDC{_^<-
    $mInnHJno~s"r<@^~H~oPsIoVmXUSp{A==1u^iYe_GeG~DQoz5"55D[D{D!1U{WOiOjB{Tw-A7xY
    ]$#OC=Z,=,TQ#*knrxAuQ*$3e~Z=_eDLi_V#(L6@,<KpJD2ZUYHRO;YFvVVJR-KR02p'x?j]]7ru
    _A}@#$s\?5>QrIwZl2_=HRnQDGn^1pju^?_?+e=?Teje$ssp{BRwv'@~sJTuWhB5Vi\XJ7_QwJrB
    5wVwIi_#wW1aoJITWj{1~UR;~uue5r;_1z=K<{Q;\O@p~5q^ZWXxRB7vBp{'CVn+RWEya[@AQ5YI
    >Y_{v3aNUjomW_XD^sKG!<UpHa{lL@j2']i5\77p[F0~YH^D\QTDG3ap12^CeE!pECDnQ*\Q[IQ[
    Gi}@5\rs=^DOG~QB\,'Ixi@3we*>Gkokp1+1gVV7CVx11M0s#=2@+*'RuQC_#lv7lI[G*}wDpu[n
    UT=;*>3BB-pnsxA\ns>1x=:>zwC1!oUwYlrlv=#G)'$*R?{o_pkaa~AAsE\lr2+vR#RC!'r}BF1{
    Y>E[-aalGmR#@j^KpXD$J3D--1,[\$kI5U+$_I]'Gu*R1I?Az>e\ZX$'iKEZE<B3K;wp~},T$~)+
    ^u!mHW\x3^pUE!xhYkDC3}7}Hv_i=}AGWrpm-5$B+[7!={81a~>X=UeC~aIODJ3,Dau=lezLO[ZE
    (Y#~3rE2T_7[^B{X,!_A2r]A7luewsm']wpx]URa*7u<*VlkJ>=1]+wuHCal!@sUU!O@YDBJELCB
    KYaRz7l<K?B-D\HpDEOx>=;w[D]?+;j>lV}X=CpQE;=B-{HnTQIT;2z=kuOiaW[*$,'}mOKzw<R?
    +ECA=<CoHI>*lEQK'x}'5][Q=AT$~#XxA2Or3o7oBj\x=CXI!@{{x~?D5+8gT*^$[l@VYJn*-[Cl
    k^Uk02j_A{n\A$KJ]s?m;BU}}wUz72Dn=1Uh1E2=$z>~}#X}12-=,JDT<a-wj25BQmu~x@*$[+5C
    vaX7y8HG^'7Rj3O;VxV1{IVZ1UCjr=zDKjCDv7?'k#~DpUU_l^!V~vIpkDx{B<rEZ3ew;oT{TK_^
    k_ciUj23U@oDU>n1s$zAO@}YkeUUE^Q25KHp'-v5<aT@p7sT[,w\^Bls+n'RYX-{pD+Wl3Dth0EW
    u5ZTX+twD<G[>V!J]_7v#Y$tEH'lKs!=e72B=XxuC~Gj!zX2=s<#j5'pH{@{C5<KBs_>6XT*]REI
    O]oBroV!EZs@QkAY*l'aAcZ+$J[4G!'umDUa7xJ{3XV~5exuv=@HhnU[>Y*CO-G2^}nGC*7J2jA^
    ,e5m#';<pWeT}YOmmkl!a2=ET?YmQymAz]C!wX^$_'"OZA,]?!D_?DooVQYOoZlvj<^>U,Qa,xJ\
    ZGUeAf<o_JM%Yps,JsnQ^d3}5\KTmI]sA<lmD'Ez+*}!+zi57HeEep-X^v-DVD2&1iBVB_v=o3W>
    b~O7XXXu'noA2zmI=]Gzoa>D$/7#AI$kHObHA[[R*A]tR3u;pEr?;1-EiQ'[@onk][*YU<e@#Dpo
    Ka-siT}GW'1JG^Cx<T~rZrED3U5][^}@;w>muD;+BK}-z?H~}+z^D^n=x2n@KDQ~=YKe1u+Jr^V]
    =DIH,]=o]J7Z&R|E_@!v5,krj[m;_ak!o+5BO?>l~G$rwUIn=$;;rpv:io_>'H@Y{''Dr#DuWlJC
    W=p~~$RjCuCuoQ1a^-vi?1$uLGD!^|WCaU,xY}^JYW;sOj_Z~TBVRZ#e$^[#7JjaX$Q]-@x<HJ7T
    EJ?s\7e?;Oj@eivvn*x=k!,WOVDZWI\B@O]YlW,!sE'Ux+N5>*aI5G~J<xEW\2D1l-/X5@[mxBx:
    <zk!XtD2>\@HV2j7
`endprotected


// *******************************************************************************************************
// **   DEBUG LOGIC                                                                                     **
// *******************************************************************************************************
// -------------------------------------------------------------------------------------------------------
//      2.01:  Memory Data Bytes
// -------------------------------------------------------------------------------------------------------

   wire [07:00] MemoryByte0000 = MemoryBlock[00000];
   wire [07:00] MemoryByte0001 = MemoryBlock[00001];
   wire [07:00] MemoryByte0002 = MemoryBlock[00002];
   wire [07:00] MemoryByte0003 = MemoryBlock[00003];
   wire [07:00] MemoryByte0004 = MemoryBlock[00004];
   wire [07:00] MemoryByte0005 = MemoryBlock[00005];
   wire [07:00] MemoryByte0006 = MemoryBlock[00006];
   wire [07:00] MemoryByte0007 = MemoryBlock[00007];
   wire [07:00] MemoryByte0008 = MemoryBlock[00008];
   wire [07:00] MemoryByte0009 = MemoryBlock[00009];
   wire [07:00] MemoryByte000A = MemoryBlock[00010];
   wire [07:00] MemoryByte000B = MemoryBlock[00011];
   wire [07:00] MemoryByte000C = MemoryBlock[00012];
   wire [07:00] MemoryByte000D = MemoryBlock[00013];
   wire [07:00] MemoryByte000E = MemoryBlock[00014];
   wire [07:00] MemoryByte000F = MemoryBlock[00015];

   wire [07:00] MemoryByte7FF0 = MemoryBlock[32752];
   wire [07:00] MemoryByte7FF1 = MemoryBlock[32753];
   wire [07:00] MemoryByte7FF2 = MemoryBlock[32754];
   wire [07:00] MemoryByte7FF3 = MemoryBlock[32755];
   wire [07:00] MemoryByte7FF4 = MemoryBlock[32756];
   wire [07:00] MemoryByte7FF5 = MemoryBlock[32757];
   wire [07:00] MemoryByte7FF6 = MemoryBlock[32758];
   wire [07:00] MemoryByte7FF7 = MemoryBlock[32759];
   wire [07:00] MemoryByte7FF8 = MemoryBlock[32760];
   wire [07:00] MemoryByte7FF9 = MemoryBlock[32761];
   wire [07:00] MemoryByte7FFA = MemoryBlock[32762];
   wire [07:00] MemoryByte7FFB = MemoryBlock[32763];
   wire [07:00] MemoryByte7FFC = MemoryBlock[32764];
   wire [07:00] MemoryByte7FFD = MemoryBlock[32765];
   wire [07:00] MemoryByte7FFE = MemoryBlock[32766];
   wire [07:00] MemoryByte7FFF = MemoryBlock[32767];

`protected
    MTI!#_{G*^zaJe_[Y/zXrCEGisghS,W7CwsRir#a1N/j=!u1*o[t5[#A,QkD*T}URDnA!{B$DIpa
    {{e2rE<xB;j\z{VC=JCVn&Z{-a~'?{IO#}GkoBC717J7V~@ozi\KBn]UAQGy_,r,DJzaN}>az7;J
    s{6]aA+^izmp_w=w]CY*m+5'+_A6r_?=Isuw]#VJMV>-W9&jEJv0u_i~p}{!={57aD#o@qZ+Z!$:
    iH=kDGAG:*A2=1un-O|ZG>x$Oi!K}Iao!a!'?2mIK=nlUrT=Y1l7<@^te)7_#!{p}]f}*OurE'*l
    E[+lbEvXsC,e{T$A{eC*V^p~\iCYn_+2nDXl{!T1@X+Qij"I<$wBC!n~U@Ou<nKb$B@Au}YxlY@K
    lTouGU[oY{<1HX_*VlBBz*D[x7IYG^o~=^vp}QQ2yo2>22wR^!U*2Z^?weV}7V)F]E[,9V<=vKnj
    iVa'~i>5=v5DprjVDzVme5mxo*<3}>TnpYs!3l{=DFn}oCO5##>=]'J1\p_<3!V'WX5Q}GftJR$3
    lE]Q$BX[O_G5|}jr1!D?WCRGiTXBxW<Ya5iZK%wB+kz}#TmrBTln][>=_#k$]3qTj]Kx1jx'WO,f
    %VhF@$Qpv@Jn&7~p7LN_s{=~=Ye?j\YWRQ@\;l?{TAYMIA3#7Hr$C?,*$T+D^(r!17ap]rEDX;=l
    #W9Vf*OR+^GkmV127^im3uTzX}Wu[mo<!Jw{lbxIu[1L+pvC?U}i}--UBH7+/o?[<}<]?G#[llul
    _j-+k{]\7_AWZXolG|LC>Z\irECLj2QpHGU[O+s7IAvJ^=O!u5kQ73<{v>wI_5*xl~ua*^+<[j-n
    C~-$U1ij*V!Tx7J@WTpu/UDl#EH]2Z&%onJJ|HaOA="s'#>u=+OVEJr]z[G#r?en*iW}?J]pjEE,
    Z<@}Bri9n\@*K'Qpl_-{}sv7vQ1^Y,R_#>'Kez#!exps'\\Yw{t1Z*i\QRn*sVj?*B^7doE{Oy&?
    \7mxEDI=\Z[A1~ReRloxBs<uEV_|vX1o7O>*WBW[oIr<}=^wxajU1^_<Kr$3HeCs2oB!q=}'EQ5'
    ?KAB}Y>$J[{x]}W{zER,;jU>;x1^i;qEFul3<!ouo#z$~w[]utYw;YQ"L=A-CxBAp\EJn!xm}[Qj
    XqZImn]H3Gzarlx1SO@WVi{^?[X~nGEric2R,;<w*2Mws@_.-11eB@Hr3nI^\,-YmIj_ARH$'Az5
    bev~U>'J-xG>G/kzvXLEV-oGmo>:.,+m\1I>\*WGvPaX-tEk$78*3jGn$ZEZU3kl6S/HpOT#sDVs
    QX$[V]1cw9vUHQ7[jWluRecUrXa{REToZ[WIm+_VwCwa,mC{n+EmeT}><vH[G]{'\xY2vsI_<Kan
    G$vW}5z-C3~b@p<In5xaD+=[LpCARJe'Ix3\#vk,!/o>J}wT[U:@{ji>{x~t7j<$}n@sI@]YWQrI
    O!]A]vJ7EAB;T^Eve!r7=zU2_+p3R1rA?w[>s*]2]?},,CY]BQapoDrkv!\pQ!5$5G}RO^Gv1KaT
    3XDaw[su]DO=jYGoKHH$wGT,[G\nQ1*{e/jD<v^[YoQ<$VI=<!C#w+U*Y]*CeU%kpCp^umTO[=WV
    B}Z12Up]<Y''nv]*+;KXAT>pwJ\~>@,A5x$}#A^GO-T?=K3lCCCs>*VBoBUoTB!B@jk$Z15QEYTG
    O=ppun;<HaC1@VC"wDD+T-oB1%#E$Y!,uj~R{15u^a1|Ik]DETo*]2sA'@7'.]~VJ[k*R#<7ALKH
    R'n\^^aTXw~T[j]}\^YTnRQKY[l3a_Gek_@jiz=@lzpGI+2=A1Wx*R{jn*vYD<@&BAQviBu>?RJ@
    CsE7G~eOR2}^onJz^_er5B2^}EJ7U$1'Vl-k<vRo}>u+Yu~OwQs_1@zzR;5T/2B\V^CmuJ}A$_2B
    V*;n7R@lYG7^31O2j^TCYT-YjAEr3uwnp$<mWljzD9GK;,TvJkOeK\uwxnpQI^JUT~RvnV[D[n=2
    s5_Y~#%$K\;v5'x;Yw,Sl!OJ}e}$;DmnDH[?OwDz;T<j=KEaD'xlXrV$/De#O_;E7Aa^5u=aRB*i
    =0En'XYA-lt#zW<LHBvQBJoET+^^{[en*<]'R12}<R!GmA7lB3CWxxm$_$oE]{I$xB-mC'pv|eJY
    $L_5#u>n[E}^{{1ToY.*Oi_s~j^NQ'H<*Dklc@,mzZR!wKr<77K>a@TsB~BzDL^iI@j#Qlia=^AO
    CG*5e]]vp;'u,i_DJ~apk1@<}<m}KDnDpp;TCExJEku_s'8W_w\2Ce}s^+kjeu?*sXoVjRisC_Vr
    []oDRK;E3RsjCi^XsX5s<\2Go'@k'YZ[me{o}j5=5BBIJ[J*}!A67WQ'5j[BHR[~p5A_O_>O!7Ba
    ,Ta*5,}a}'O,K];aW-;C*m,{7JQTIs}X?R@o'X\^X1\[C2A,eO>AdE[#JEZx+7_o~V=enKEw$x+^
    O*^BT<[MxmXmYl~;W+Y3o[^s7<\m2GzsualaUQ5<^HCY3x3=1+m{xi<EK8/I;_k=+,eQ_3uk<Q7b
    FIY#kk+YkURRiH>~D3]@2x}l'tx+ep!D!*l[olkpMXas+zaEG=ZYO3Ci>zH-u}QiE7u<OkI=Cs?V
    eB\{,aDYiZn<3}_o7{AB]_o{s"[vi*3YJmZXA'JaU~];U7pe^;5R3km+n]})~'wX/D*JrJ{1TVJe
    R$'=^!EQsn{Js-opnQb>X'sw5En+<eZ&gdzpH_-Cl5aOK2jxVJJ[},}w{ww{Tu?Cr3^1KK2]$k%v
    3]XV<-O$7x@EeWw,m7@j,VUv{7{Y<;\CGR'5ROl^[J~2}276?a7O^k-?[uTr,IlwK$DuTRA,YU2^
    {1XnCZ1?-X,~Jo;,?GT#[v5E$3;X;RBE#Oxj;=wZT]nsC1C?'[Z;RDvkE@BBT]aIkIik@1lKO5<z
    c_*'Jsi>z>[+!7]+e|?7Gv[<!>*J5rQ7;Yjm<KA+oE~<xD7<^{,HA{1p;{3I+Q?7^BJIvV]3^$H=
    {5[+auR2{'bB~1X{Xnex[K=5kOteO3KQPasCXe_]up3_o!Xv#kx-j5^7u]?ssr^v7}T=,DZBKs{e
    1xwv=C![C;X5^J,zm1?E@cx+=_21i2*kwvHekluH\a$^W>XIa^D3ZUt}x?1E<rz!TQ@>{m#w>2w>
    G?BX7kB+_>[R<5W1wHVeD+Un{-7j,_X.9I'r2Iwz3oqYl-]2oO5$vl!%l*K2v#u-E3Em1oC;}M=>
    'KP$^AJA=VmT*kQEXw}z!YB*+CrIml_Y|Wro-m{*XjE#7Y'B_m7mX!},7z@!CR[1][noi92EoIlT
    a_{jKzkY@YEKpl=!lG*#A^]}Ep6Jo~[OmV}1EKBc
`endprotected


   specify
         specparam
`ifdef tCLK4300
            tSP  =  0.65,                                  // Input setup time
            tHD  =  0.65,                                  // Input hold time
	    tDS  =  0.65,
	    tDH  =  0.65,
	    tCSP =  1.6,
	    tCHD =  1.6;
`else
	    tSP  =  0.85,                                  // Input setup time
            tHD  =  0.85,                                  // Input hold time
	    tDS  =  0.85,
	    tDH  =  0.85,
	    tCSP =  2,
	    tCHD =  2;
`endif

      $setuphold(xCLK, xDQ &&& TimingCheckAddrEnable, tSP, tHD, sys_timing_err_input);
      
      $setuphold  (posedge xCLK, xCEn, tCSP, tCHD, sys_timing_err_CE);
      
      $setuphold(xDQS, xDQ &&& TimingCheckDataEnable, tDS, tDH, sys_timing_err_data);
      $setuphold(xDQS, xDM &&& TimingCheckDMEnable, tDS, tDH, sys_timing_err_data);
      
  endspecify
  
  always @(sys_timing_err_input) ERROR ("tSP/tHD Violation");
  always @(sys_timing_err_CE) ERROR ("tCSP/tCHD Violation");
  always @(sys_timing_err_data) ERROR ("tDS/tDH Violation");

endmodule
